(ns com.aurellem.run.title
  (:use (com.aurellem.gb gb-driver vbm)))

(defn first-difference [base alt summary root]
  (loop [branch-point root
         actions []]
    (let [base-branch (step branch-point base)
          base-val (summary base-branch)
          alt-branch (step branch-point alt)
          alt-val (summary alt-branch)]
      (if (not= base-val alt-val)
        [(conj actions alt) alt-branch]
        (recur base-branch (conj actions base))))))

(defn advance
  ([base alt summary [commands state]]
     (let [[c s] (first-difference base alt summary state)]
       [(concat commands c) s]))
  ([base alt [commands state]]
     (advance base alt AF [commands state]))
  ([alt [commands state]]
     (advance [] alt [commands state])))

(def scroll-text (partial advance [:b] [:a :b]))

(defn start [] [[] (root)])

(defn-memo title
  ([] (title (start)))
  ([script]
     (->> script
          (advance [] [:a])
          (advance [] [:start])
          (advance [] [:a])
          (advance [] [:start]))))

(defn-memo oak
  ([] (oak (title)))
  ([script]
     (->> script
          scroll-text
          scroll-text
          scroll-text
          scroll-text
          scroll-text
          scroll-text
          scroll-text
          scroll-text
          scroll-text
          scroll-text
          scroll-text
          scroll-text
          scroll-text)))

(defn-memo name-entry-rlm
  ([] (name-entry-rlm (oak)))
  ([script]
     (->> script
          (advance [] [:a])
          (advance [] [:r] DE)
          (play-moves
           [[]
            [:r] [] [:r] [] [:r] [] [:r] []
            [:r] [] [:r] [] [:r] [] [:d] [:a]  
            [:l] [] [:l] [] [:l] [] [:l] []
            [:l] [] [:l] [:a] [] [:r] [:a]
            [:r] [] [:r] [] [:r] [] [:r] []
            [:r] [] [:d] [] [:d] [] [:d] [:a]]))))

(defn-memo name-entry-ash
  ([] (name-entry-ash (oak)))
  ([script]
     (->> script
          (advance [] [:d])
          (advance [] [:d])
          (advance [] [:a]))))
       
(defn-memo rival-name-entry-gary
  ([] (rival-name-entry-gary (name-entry-ash)))
  ([script]
     (->> script
          scroll-text
          scroll-text
          scroll-text
          scroll-text
          scroll-text
          (advance [] [:d])
          (advance [] [:d])
          (advance [] [:a]))))
  
(defn-memo rival-name-entry-blue
  ([] (rival-name-entry-blue (name-entry-ash)))
  ([script]
     (->> script
          scroll-text
          scroll-text
          scroll-text
          scroll-text
          scroll-text
          (advance [] [:d])
          (advance [] [:a]))))

(defn-memo finish-title
  ([] (finish-title (rival-name-entry-blue)))
  ([script]
     (->> script
          scroll-text
          scroll-text
          scroll-text
          scroll-text
          scroll-text
          scroll-text
          scroll-text)))

(def title-frames 2194)

(defn title-checkpoint! []
  (let [[moves state] (finish-title)]
    (assert (= title-frames (count moves)))
    [(write-moves! moves "title-checkpoint")
     (write-state! state "title-checkpoint")]))

(defn intro []
  [(read-moves "title-checkpoint")
   (read-state "title-checkpoint")])

(defn test-intro []
  (play-vbm (moves-filename "title-checkpoint")))

;; TODO might be able to glue these together more elegantly with monads

